package org.sourceforge.jbezier;

/**
 * This class provides functionality to generate a bzier curve of any order (See "important note.txt" )
 * @author Marius Brandt(@PrydeRage)
 *
 */
public class JBezier {
	private int order;
	private Point[] srcPoints;
	public Point[] bPoints;
	private double fidelty;
	
	
	/**
	 * Initializes all important values for use with bzier curves
	 * @param order The most common orders, which can be found in BezierOrder.java
	 * @param iterations How many points should be generated. 
	 * More points = higher quality but slower computing speed; 
	 * Less points = lower quality but faster computing speed
	 * @param points Any amount of vertices of the type Point (can be an array)
	 */
	public JBezier(BezierOrder order, int iterations, Point ...points){
		if(order.order != points.length - 1){
			return;
		}
		this.order = order.order;
		srcPoints = points;
		fidelty = iterations;
		bPoints = new Point[(int) fidelty+1];
	}
	
	/**
	 * Initializes all important values for use with bzier curves
	 * @param order Can be any integer (See "important note.txt" for details). The order is the amount of points minus 1
	 * @param iterations How many points should be generated. 
	 * More points = higher quality but slower computing speed; 
	 * Less points = lower quality but faster computing speed
	 * @param points Any amount of vertices of the type Point (can be an array)
	 */	
	public JBezier(int order, int iterations, Point ...points){
		if(order != points.length - 1){
			return;
		}
		this.order = order;
		srcPoints = points;
		fidelty = iterations;
		bPoints = new Point[(int) fidelty+1];
	}
	
	/**
	 * Generates the bezier curve and stores the individual points into an array.The amount of points is determined by the number of iterations. (e.g. 1000 iterations result in an array of type Point[] from 0 - 999)
	 * @return Point[] - Array of the calculated points. 
	 */
	public Point[] generateBezier(){
		int pCounter = 0;
		int numPoints = order+1;
		for(double t = 0; t <= 1; t += (1 / fidelty)){
			double x=0;
			double y=0;

			for(int p = 0; p < numPoints; p++){
				x += utils.binominalCoefficient(order, p) * Math.pow(t, p) * Math.pow(1-t, order-p) * srcPoints[p].x;
				y += utils.binominalCoefficient(order, p) * Math.pow(t, p) * Math.pow(1-t, order-p) * srcPoints[p].y;
			}
			
			bPoints[pCounter] = new Point(x, y);
			++pCounter;
		}
		return bPoints;
	}
	
	/**
	 * Sets a new order for the bzier curve that should be used when calling generateBezier() again (Should only be used in conjunction with the update() function).
	 * @param neworder The new order that should be used when generateBezier is called again
	 */
	public void setOrder(int neworder){
		order = neworder;
	}
	
	/**
	 * Returns the current order of the bzier curve
	 * @return int - current order of the bzier curve
	 */
	public int getOrder(){
		return order;
	}
	
	/**
	 * 	Generates a new bzier based on the new points supplied (This takes changes of order into calculation).
	 * @param points The points that should make up the new bzier curve.
	 * @return Point[] - Array of the calculated points. 
	 */
	public Point[] update(Point ...points){
		srcPoints = points;
		return generateBezier();
	}
	
}
